from __future__ import annotations

import sys

from astroid import nodes

from pylint.checkers import utils
from pylint.checkers.base.basic_checker import _BasicChecker

if sys.version_info >= (3, 8):
    from typing import Literal
else:
    from typing_extensions import Literal

def _infer_dunder_doc_attribute(
    node: nodes.Module | nodes.ClassDef | nodes.FunctionDef,
) -> str | None:
    # Try to see if we have a `__doc__` attribute.
    try:
        docstring = node["__doc__"]
    except KeyError:
        return None

    docstring = utils.safe_infer(docstring)
    if not docstring:
        return None
    if not isinstance(docstring, nodes.Const):
        return None
    return str(docstring.value)

class ConfluencePageChecker(_BasicChecker):
    name = 'missing-documentation-link'
    priority = -1
    msgs = {
        'C4001': (
            'Module docstring must link to a *.atlassian.net page.',
            'missing-documentation-link',
            'This module is missing a link to a Confluence documentation page. Each module should have a corresponding documentation page.'
        ),
    }

    def visit_module(self, node: nodes.Module) -> None:
        self._check_documentation_link(node)

    def _check_documentation_link(self, node: nodes.Module, report_missing: bool = True) -> None:
        docstring = node.doc_node.value if node.doc_node else None
        if docstring is None:
            docstring = _infer_dunder_doc_attribute(node)
        if docstring is None:
            self.add_message('missing-documentation-link', node=node)
            if not report_missing:
                return
        elif "atlassian.net" not in docstring:
            self.add_message('missing-documentation-link', node=node)



##
# Copyright 2020 Palantir Technologies, Inc. All rights reserved.
# Licensed under the MIT License (the "License"); you may obtain a copy of the
# license at https://github.com/palantir/pyspark-style-guide/blob/develop/LICENSE
##


import astroid

from pylint.checkers import BaseChecker
from pylint.interfaces import IAstroidChecker

from .pylint_utils import compute_arguments_length
from .pylint_utils import is_line_split
from .pylint_utils import get_length


class StatementCallChecker(BaseChecker):
    __implements__ = IAstroidChecker

    name = "split-statement"
    priority = -1
    msgs = {
        "E1073": (
            "Statements should be on a single line if they fit.",
            "unnecessarily-split-statement",
            "This statement fits into a single line and as such should not be split across multiple lines.",
        ),
    }

    def is_line_split(self, function):
        line = function.lineno
        for arg in function.args.args:
            if arg.lineno != line:
                return True
        return False

    def __init__(self, linter=None):
        super(StatementCallChecker, self).__init__(linter)
        self._function_stack = []

    def visit_functiondef(self, node):
        self._function_stack.append([])

    def visit_assign(self, node):
        if is_line_split(node):
            base_length = 3  # _=_
            if get_length(node) + base_length <= 120:
                self.add_message("unnecessarily-split-statement", node=node)

    def leave_functiondef(self, node):
        self._function_stack.pop()



##
# Copyright 2020 Palantir Technologies, Inc. All rights reserved.
# Licensed under the MIT License (the "License"); you may obtain a copy of the
# license at https://github.com/palantir/pyspark-style-guide/blob/develop/LICENSE
##


import astroid

from pylint import checkers
from pylint import interfaces


class ChainedDotFunctionsSyntaxChecker(checkers.BaseChecker):
    __implements__ = interfaces.IAstroidChecker

    name = 'chained-function-length'

    msgs = {
        'E1085': (
            'Chained functions applied on a variable should not be more than 5.',
            'chained-function-length',
            'Applied functions should not be more than 5 as it becomes difficult to follow.'
        ),
    }

    def visit_assign(self, node):
        depth = get_num_of_first_level_functions(node, 0)
        if depth > 5:
            self.add_message('chained-function-length', node=node)
        else:
            return


def get_num_of_first_level_functions(node, counter):
    if hasattr(node, 'value'):
        curr_value = node.value
        if isinstance(curr_value, astroid.Call):
            return get_num_of_first_level_functions(curr_value.func, counter+1)
    elif hasattr(node, 'expr'):
        curr_value = node.expr
        if isinstance(curr_value, astroid.Call):
            return get_num_of_first_level_functions(curr_value.func, counter+1)

    return counter




import astroid
from astroid import nodes
from pylint.checkers import BaseChecker
from pylint.interfaces import IAstroidChecker
from pylint.checkers.imports import ImportsChecker

class CaseImportChecker(BaseChecker):
    __implements__ = IAstroidChecker

    name = "import-case-checker"

    priority = -1
    msgs = {
        'C4003': ('pyspark.sql.functions module should be imported as F instead of f',
                  'lower-case-function-import',
                  'Follow AstrumU convention by importing the functions module using capital F.'),
        # 'C4004': ('pyspark.sql.types module should be imported as T instead of t',
        #           'lower-case-type-import',
        #           'Follow AstrumU convention by importing the types module using capital T.'),
    }
    def __init__(self, linter):
        super(CaseImportChecker, self).__init__(linter)
        self._check_import_stack = []

    def _check_import(self, node: nodes.Import):
        print(node.names[0][0])
        print(node.names[0][1])
        if node.names[0][0] == "pyspark.sql.function" and node.names[0][1] == 'f':
            self.add_message(
                "lower-case-function-import", node=node
            )
        # elif node.names[0][0] == "pyspark.sql.types" and node.names[0][1] == 't':
        #     self.add_message(
        #         "lower-case-type-import", node=node
        #     )

    def visit_import(self, node):
        self._check_import_stack.append([])
        

    def leave_import(self, node):
        self._check_import(node)
        self._check_import_stack.pop()